@@ -64,8 +64,8 @@ class Neural_NMF(nn.Module):
         if c is not None:
             self.linear = nn.Linear(depth_info[-1],c, bias = False).double() 
                                         #initialize classification layer (with last factor matrix)
-            with torch.no_grad():
-                self.linear.weight.copy_(torch.abs(torch.rand(depth_info[-1],c)))
+            #with torch.no_grad():
+            #    self.linear.weight.copy_(torch.abs(torch.rand(depth_info[-1],c)))
 
 
     def forward(self, X, Y=None, L=None, last_S_lst=None):
@@ -104,7 +104,8 @@ class Neural_NMF(nn.Module):
 
             #B = torch.mm(Y[:,L[0]==1], torch.pinverse(X[:,L[0]==1]))
             
-            #B = torch.mm(Y, torch.pinverse(X))
+            B = torch.mm(Y, torch.pinverse(X))
+            self.linear.weight = B
             #pred = torch.transpose(self.linear(torch.transpose(X,0,1)),0,1)
             #pred = torch.mm(B,X)
             pred = self.linear(X) #Jamie testing
