@@ -87,7 +87,7 @@ def train(net, X, loss_func="Recon Loss", supervised=False, label=None, L=None,
             raise ValueError("Expected a two-dimensional Tensor, but X is of shape {}".format(X.shape))
             
     if optimizer != "gd" and optimizer != "adam":
-        raise ValueError("Expected 'gd' of 'adam' for optimizer, but optimizer parameter is {}".format(optimizer))
+        raise ValueError("Expected 'gd' or 'adam' for optimizer, but optimizer parameter is {}".format(optimizer))
 
     if supervised == False:
 
@@ -255,7 +255,7 @@ def train_supervised(net, X, label, L = None, loss_func="Recon Loss", epoch = 10
         Indicator for whether to initialize the estimated support for the least squares solution
         based on the previous support, when using the fnnls method
     optimizer: string, optional
-        Optimizer to use to optimize Neural NMF model, either 'gd' for gradient descent of 'adam' for adam optimizer
+        Optimizer to use to optimize Neural NMF model, either 'gd' for gradient descent or 'adam' for adam optimizer
     verbose: bool, optional
         Indicator for whether to print the loss every verbose_epoch epochs (default True).
     verbose_epoch: int_, optional
@@ -275,6 +275,13 @@ def train_supervised(net, X, label, L = None, loss_func="Recon Loss", epoch = 10
 
     """
     
+    if len(lr) == 2:
+        lr_nmf = lr[0]
+        lr_class = lr[1]
+    else:
+        lr_nmf = lr
+        lr_class = lr
+    
     if optimizer != "gd" and optimizer != "adam":
         raise ValueError("Expected 'gd' of 'adam' for optimizer, but optimizer parameter is {}".format(optimizer))
 
@@ -289,7 +296,7 @@ def train_supervised(net, X, label, L = None, loss_func="Recon Loss", epoch = 10
         config_w = {}
 
         for config in configs:
-            config['learning_rate'] = lr
+            config['learning_rate'] = lr_nmf
             config['beta1'] = 0.9
             config['beta2'] = 0.99
             config['epsilon'] = 1e-8
@@ -347,15 +354,44 @@ def train_supervised(net, X, label, L = None, loss_func="Recon Loss", epoch = 10
             if(optimizer == "adam"):
                 A.data, configs[l] = adam(A.data, A.grad.data, configs[l])
             else:
-                A.data = A.data.sub_(lr*A.grad.data)
+                A.data = A.data.sub_(lr_nmf*A.grad.data)
 
             A.data = A.data.clamp(min = 0)
             
+        # doing gradient update for classification layer
+        for iter_classifier in range(30):
+            net.zero_grad()
+            if S_lst != None and initialize_support:
+                S_lst, pred = net(X, Y=Y, L=L, last_S_lst=[s.detach() for s in S_lst])
+            else:
+                S_lst, pred = net(X, Y=Y, L=L)
+
+            loss = None
+        
+            if type(loss_func) == str:
+                loss_nmf = loss_functions[loss_func](net, X, S_lst)
+            else:
+                loss_nmf = loss_func(net, X, S_lst)
+         
+            if type(loss_func) == str:
+                    loss = loss_functions[loss_func](net, X, S_lst,pred,label,L)
+            else:
+                loss = loss_func(net, X, S_lst,pred,label,L)
+            
+            loss.backward()
+            S_lst[0].detach()
+            history.add_scalar('loss',loss.data)
+            weight = net.linear.weight
+            weight.data = weight.data.sub_(lr_class*weight.grad.data)
+            history.add_tensor('weight', weight.data.clone())
+            history.add_tensor('grad_weight', weight.grad.data.clone())
+            
         
         #if(verbose) and (i+1)%verbose_epoch == 0:
         #    print('epoch = ', i+1, '\n', loss.data)
         if (i+1)%decay_epoch == 0:
-            lr = lr*weight_decay
+            lr_nmf = lr_nmf*weight_decay
+            lr_class = lr_class*weight_decay
 
     if full_history:        
         return history
